//
//  StatusTableViewCell.swift
//  Tusker
//
//  Created by Shadowfacts on 8/16/18.
//  Copyright © 2018 Shadowfacts. All rights reserved.
//

import UIKit
import MastodonKit

protocol StatusTableViewCellDelegate {
    
    func selected(account: Account)
    
    func selected(mention: Mention)
    
    func selected(tag: MastodonKit.Tag)
    
    func selected(url: URL)
    
    func selected(status: Status)
    
    func reply(to status: Status)
    
}

class StatusTableViewCell: UITableViewCell, PreferencesAdaptive {

    var delegate: StatusTableViewCellDelegate?
    
    @IBOutlet weak var displayNameLabel: UILabel!
    @IBOutlet weak var usernameLabel: UILabel!
    @IBOutlet weak var contentLabel: StatusContentLabel!
    @IBOutlet weak var avatarImageView: UIImageView!
    @IBOutlet weak var reblogLabel: UILabel!
    
    var status: Status!
    var account: Account!
    var reblogger: Account?
    
    var avatarURL: URL?
    
    override func awakeFromNib() {
        displayNameLabel.addGestureRecognizer(UITapGestureRecognizer(target: self, action: #selector(accountPressed)))
        displayNameLabel.isUserInteractionEnabled = true
        usernameLabel.addGestureRecognizer(UITapGestureRecognizer(target: self, action: #selector(accountPressed)))
        usernameLabel.isUserInteractionEnabled = true
        reblogLabel.addGestureRecognizer(UITapGestureRecognizer(target: self, action: #selector(reblogLabelPressed)))
        reblogLabel.isUserInteractionEnabled = true
        avatarImageView.addGestureRecognizer(UITapGestureRecognizer(target: self, action: #selector(accountPressed)))
        avatarImageView.isUserInteractionEnabled = true
        avatarImageView.layer.masksToBounds = true
    }
    
    func updateUIForPreferences() {
        avatarImageView.layer.cornerRadius = Preferences.shared.avatarStyle.cornerRadius(for: avatarImageView)
        if let reblogger = reblogger {
            reblogLabel.text = "Reblogged by \(reblogger.realDisplayName)"
        }
        displayNameLabel.text = account.realDisplayName
    }
    
    func updateUI(for status: Status) {
        self.status = status
        
        let account: Account
        if let reblog = status.reblog {
            account = reblog.account
            reblogger = status.account
            reblogLabel.isHidden = false
        } else {
            account = status.account
            reblogLabel.isHidden = true
            reblogger = nil
        }
        self.account = account
        
        
        updateUIForPreferences()
        
        usernameLabel.text = "@\(account.acct)"
        avatarImageView.image = nil
        if let url = URL(string: account.avatar) {
            avatarURL = url
            AvatarCache.shared.get(url) { image in
                DispatchQueue.main.async {
                    self.avatarImageView.image = image
                    self.avatarURL = nil
                }
            }
        }
        
        contentLabel.status = status
        contentLabel.delegate = self
    }
    
    override func prepareForReuse() {
        if let url = avatarURL {
            AvatarCache.shared.cancel(url)
        }
    }

    @IBAction func replyPressed(_ sender: Any) {
        delegate?.reply(to: status)
    }
    
    @objc func accountPressed() {
        delegate?.selected(account: account)
    }
    
    @objc func reblogLabelPressed() {
        guard let reblogger = reblogger else { return }
        delegate?.selected(account: reblogger)
    }
    
    func didSelect() {
        delegate?.selected(status: status)
    }
    
}

extension StatusTableViewCell: HTMLContentLabelDelegate {
    
    func selected(mention: Mention) {
        delegate?.selected(mention: mention)
    }
    
    func selected(tag: MastodonKit.Tag) {
        delegate?.selected(tag: tag)
    }
    
    func selected(url: URL) {
        delegate?.selected(url: url)
    }
    
}
